'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const is_1 = require("../utils/is");
const action_1 = require("./action");
const storeTracker_1 = require("./storeTracker");
const memoizedApplyDeriveFactory = () => {
    let _target = undefined;
    return function applyDerive(derive, target) {
        if (target === _target)
            return target;
        if (is_1.isFunction(derive)) {
            const derived = derive(target);
            _target = derived;
            return derived;
        }
        else {
            return target;
        }
    };
};
const call = (target, ...args) => {
    if (typeof target === 'function')
        target = [null, target];
    const [context, fn] = target;
    return fn.apply(context, args);
};
exports.reducerFactory = ({ nsp, path, params, methods, getContext, derive, defaultState }) => {
    const memoizedApplyDerive = memoizedApplyDeriveFactory();
    const reducer = (state, action, env) => {
        // see if we hold a method that match current action
        const { methodName, namespace, effectivePath } = env;
        const method = methods[methodName];
        const shouldBypass = namespace !== nsp || !is_1.isFunction(method);
        if (shouldBypass)
            return state;
        // if finalize, simply return finalized state
        if (action.meta && action.meta.finalize)
            return action.payload;
        const context = getContext(state, effectivePath, env);
        return call([context, method], state, action);
    };
    const facadeReducer = (state = defaultState, action, store) => {
        if (store)
            storeTracker_1.activateStore(store);
        // parse action, prepare metadata
        const parsed = action_1.parseAction(action) || {};
        const { namespace, method: methodName, path: actionPath } = parsed;
        const effectivePath = params ? path : actionPath;
        const runtimeEnv = Object.assign({}, store, { effectivePath, namespace, methodName });
        // invoke the working reducer
        const nextState = reducer(state, action, runtimeEnv);
        // invoke done, reset env
        if (store)
            storeTracker_1.deactivateStore(store);
        if (is_1.isIterator(nextState)) {
            // this is an effect, just return it to service gateway to run it.
            return nextState;
        }
        else {
            // plain object state, apply the derive;
            return derive ? memoizedApplyDerive(derive, nextState) : nextState;
        }
    };
    return facadeReducer;
};
